#ifndef INC_INIFILEERRORS_H
#define INC_INIFILEERRORS_H
//-----------------------------------------------------------------------------
/*!
\file IniFileErrors.h
\brief
\if english
  For each library, here 'IniFile' there is an error file.
\endif

\if german
  Für jede Bibliothek, hier 'IniFile' gibt es eine Fehler-Datei.
\endif
    
\author Helmut Jakoby
\copyright © 1995 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the IniFile module.

IniFile is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General
Public License as published by the Free Software Foundation; either version 3 of
the License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
namespace ini
{
  //---------------------------------------------------------------------------
  /*!
  \enum ERRORS
  \brief
  \if english
    Error range = -15501 to -15600
  \endif

  \if german
    Fehlerbereich = -15501 bis -15600
  \endif
  */
  //---------------------------------------------------------------------------
  enum ERRORS
  {
    //-------------------------------------------------------------------------
    /*! -15501: \if english
                  An ini-file could not be opened or read
                \endif
              
                \if german
                  Ein Ini-File konnte nicht geöffnet bzw. gelesen werden
                \endif
    */
    READ_ERROR = -15501,
    //-------------------------------------------------------------------------
    /*! -15502: \if english
                  An ini-file could not be written
                \endif
              
                \if german
                  Ein Ini-File konnte nicht geschrieben werden
                \endif
    */
    WRITE_ERROR = -15502,
    //-------------------------------------------------------------------------
    /*! -15503: \if english
                  An invalid parameter or #NULL_PTR was passed
                \endif
                \if german
                  Es wurde ein ungültiger Parameter bzw. #NULL_PTR überegeben
                \endif
    */
    PARAMETER_NULL_FORBIDDEN = -15503,
    //-------------------------------------------------------------------------
    /*! -15504: \if english
                  No object could be created
                \endif
              
                \if german
                  Es konnte kein Objekt kreiert werden
                \endif
    */
    CREATE= -15504,
    //-------------------------------------------------------------------------
    /*! -15505: \if english
                  No valid conversion of an object could be performed
                \endif
              
                \if german
                  Es konnte keine gültige Umwandlung eines Objektes durchgeführt werden
                \endif
    */
    CAST = -15505,
    //-------------------------------------------------------------------------
    /*! -15506: \if english
                  The searched key could not be found
                \endif
              
                \if german
                  Es konnte der gesuchte Key nicht gefunden werden
                \endif
    */
    KEY_NOT_EXISTING = -15506,
    //-------------------------------------------------------------------------
    /*! -15507: \if english
                  The searched section could not be found
                \endif
              
                \if german
                  Es konnte die gesuchte Sektion nicht gefunden werden
                \endif
    */
    SECTION_NOT_EXISTING = -15507,
    //-------------------------------------------------------------------------
    /*! -15508: \if english
                  A section to be inserted already exists
                \endif
              
                \if german
                  Eine einzufügende Sektion ist schon vorhanden
                \endif
    */
    SECTION_ALLREADY_EXISTING = -15508,
    //-------------------------------------------------------------------------
    /*! -15509: \if english
                  An expected file name was not passed
                \endif
                \if german
                  Ein erwarteter Dateiname wurde nicht übergeben
                \endif
    */
    NO_FILENAME = -15509,
    //-------------------------------------------------------------------------
    /*! -15600: \if english
                  Unknown error
                \endif
              
                \if german
                  Unbekannter Fehler
                \endif
    */
    UNKNOWN = -15600
    //-------------------------------------------------------------------------
  };
  //---------------------------------------------------------------------------
} // namespace ini
//-----------------------------------------------------------------------------
#endif
