//-----------------------------------------------------------------------------
/*!
\file IniKey.cpp
\brief Source for Key
\author Helmut Jakoby
\copyright © 1995 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the IniFile module.

IniFile is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General
Public License as published by the Free Software Foundation; either version 3 of
the License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "IniKey.h"
//-----------------------------------------------------------------------------
namespace ini
{
  //---------------------------------------------------------------------------
  #define SUPER Line
  //---------------------------------------------------------------------------
  /*! This function or variable may be unsafe. Consider using strcpy_s instead. To
  disable deprecation, use _CRT_SECURE_NO_WARNINGS. See online help for details. */
  #ifdef _MSC_VER
    #pragma warning( disable : 4996 )
  #endif
  //---------------------------------------------------------------------------
  Key::Key( const std::string & sCharLine )
  : SUPER(sCharLine, IniVARIABLE)
  {
  }
  //---------------------------------------------------------------------------
  Key::~Key ()
  {
  }
  //---------------------------------------------------------------------------
  Key::Key( const std::string & sCharLine, LINE_TYPE eLineType )
  : SUPER(sCharLine, eLineType)
  {
  }
  //---------------------------------------------------------------------------
  Key::Key()
  {
  }
  //---------------------------------------------------------------------------
  std::string Key::getKeyName() const
  {
    std::size_t   t_Position = m_sText.find( "=" );

    if ( t_Position != std::string::npos )
    {
      std::string   t_sRetVal = m_sText;

      t_sRetVal.erase( t_Position, std::string::npos );
      return t_sRetVal;
    }
    return m_sText;
  }

  //-------------------------------------------------------------------
  void Key::setKeyName( const std::string & sName )
  {
    if ( ! sName.empty() )
    {
      std::size_t   t_Position = m_sText.find( "=" );

      m_sText.replace( 0, t_Position - 1, sName );
    }
  }
  //---------------------------------------------------------------------------
  std::string Key::getKeyValue () const
  {
    std::string   t_sRetVal = m_sText;

    if ( ! t_sRetVal.empty() )
    {
      std::size_t   t_Position = t_sRetVal.find( "=" );

      t_sRetVal.erase( 0, t_Position + 1 ); // ohne "="
      return t_sRetVal;
    }
    return m_sText;
  }
  //---------------------------------------------------------------------------
  void Key::setKeyValue ( const std::string & sValue )
  {
    std::size_t   t_Position = m_sText.find( "=" );
  
    m_sText.replace( t_Position + 1, std::string::npos, sValue );
  }
  //---------------------------------------------------------------------------
  /*! This function or variable may be unsafe. Consider using strcpy_s instead. To
  disable deprecation, use _CRT_SECURE_NO_WARNINGS. See online help for details. */
  #ifdef _MSC_VER
    #pragma warning( default : 4996 )
  #endif
  //---------------------------------------------------------------------------
  #undef SUPER
  //---------------------------------------------------------------------------
} // namespace ini
