#ifndef INC_EUSTDFUNC_H
#define INC_EUSTDFUNC_H
//-----------------------------------------------------------------------------
/*!
\file EuStdFunctions.h
\brief
\if english
  Header for StdStringFunctions
\endif

\if german
  Header für StdStringFunctions
\endif
\author Helmut Jakoby
\copyright © 2010 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the EuTools module.

EuTools is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include <EuToolTypes.h>
#include "EuException.h"
#include <iomanip>      // std::setw
#include <string>
#include <vector>
#include <set>
#if defined (_MSC_VER) || defined(__MINGW32__)
  #include <windows.h>
  //#include <codecvt> // warum das?
#endif
#if defined(__MINGW32__) || defined (__GNUC__)
  #include <cstring>
#endif
#include <ctime>
//-----------------------------------------------------------------------------
namespace eut
{
  //---------------------------------------------------------------------------
  /*!
  \brief
  \if english
    Help functions for the processing of strings.
  \endif

  \if german
    Hilfsfunktionen für die Verarbeitung von Strings.
  \endif
  \copyright © 2010 Helmut Jakoby
  \author Helmut Jakoby
  */
  namespace StdStringFunctions
  {
    //-------------------------------------------------------------------------
    /*!
      \if english
        Color codes for std::cout output in the console.
      \endif
    
      \if german
        Farbcodes für std::cout Ausgaben in der Konsole.
      \endif
    */
    enum EnCoutColor
    {
      eCOLOR_DEFAULT = 0,
      eCOLOR_BLACK = 1,
      eCOLOR_BLUE = 2,
      eCOLOR_GREEN = 3,
      eCOLOR_LIGHTBLUE = 4,
      eCOLOR_RED = 5,
      eCOLOR_MAGENTA = 6,
      eCOLOR_YELLOW = 7,
      eCOLOR_LIGHTGRAY = 8,
      eCOLOR_GRAY = 9,
      eCOLOR_WHITE = 10
    };

    /*
    typedef unsigned int EuCoutColor;
    const EuCoutColor EuFOREGROUND_BLACK = 0;       //!< MS: xxxx 0000   Linux: \033[0;30;??m
    const EuCoutColor EuFOREGROUND_BLUE = 1;        //!< MS: xxxx 0001   Linux: \033[0;34;??m
    const EuCoutColor EuFOREGROUND_GREEN = 2;       //!< MS: xxxx 0010   Linux: \033[0;32;??m
    const EuCoutColor EuFOREGROUND_LIGHTBLUE = 3;   //!< MS: xxxx 0011   Linux: \033[0;36;??m
    const EuCoutColor EuFOREGROUND_RED = 4;         //!< MS: xxxx 0100   Linux: \033[0;31;??m
    const EuCoutColor EuFOREGROUND_MAGENTA = 5;     //!< MS: xxxx 0101   Linux: \033[0;35;??m
    const EuCoutColor EuFOREGROUND_YELLOW = 6;      //!< MS: xxxx 0110   Linux: \033[0;33;??m
    const EuCoutColor EuFOREGROUND_LIGHTGRAY = 7;   //!< MS: xxxx 0111   Linux: \033[0;39;??m
    const EuCoutColor EuFOREGROUND_GRAY = 8;        //!< MS: xxxx 1000   Linux: \033[0;37;??m
    const EuCoutColor EuFOREGROUND_WHITE = 15;      //!< MS: xxxx 1111   Linux: \033[0;39;??m

    const EuCoutColor EuBACKGROUND_BLACK = 0;       //!< MS: 0000 xxxx   Linux: \033[0;??;40m
    const EuCoutColor EuBACKGROUND_BLUE = 16;       //!< MS: 0001 xxxx   Linux: \033[0;??;44m
    const EuCoutColor EuBACKGROUND_GREEN = 32;      //!< MS: 0010 xxxx   Linux: \033[0;??;42m
    const EuCoutColor EuBACKGROUND_LIGHTBLUE = 48;  //!< MS: 0011 xxxx   Linux: \033[0;??;46m
    const EuCoutColor EuBACKGROUND_RED = 64;        //!< MS: 0100 xxxx   Linux: \033[0;??;41m
    const EuCoutColor EuBACKGROUND_MAGENTA = 80;    //!< MS: 0101 xxxx   Linux: \033[0;??;45m
    const EuCoutColor EuBACKGROUND_YELLOW = 96;     //!< MS: 0110 xxxx   Linux: \033[0;??;43m
    const EuCoutColor EuBACKGROUND_LIGHTGRAY = 112; //!< MS: 0111 xxxx   Linux: \033[0;??;47m
    const EuCoutColor EuBACKGROUND_GRAY = 128;      //!< MS: 1000 xxxx   Linux: \033[0;??;47m
    const EuCoutColor EuBACKGROUND_WHITE = 240;     //!< MS: 1111 xxxx   Linux: \033[0;??;47m
    */
    //-------------------------------------------------------------------------
    /*!
      \if english
        Positioning in the string for StdStringFunctions::fillString(std::string&, char, std::size_t, EnStrPos)
      \endif
    
      \if german
        Positionierung im String für StdStringFunctions::fillString(std::string&, char, std::size_t, EnStrPos)
      \endif
    */
    enum EnStrPos
    {
      /*!
      \if english
        It should be inserted or removed at the beginning.
      \endif
    
      \if german
        Es soll am Anfang eingefügt bzw. entfernt werden.
      \endif
      */
      eBEFORE,
      /*!
      \if english
        It should be inserted or removed at the end.
      \endif
    
      \if german
        Es soll am Ende eingefügt bzw. entfernt werden.
      \endif
      */
      eBEHIND,
      /*!
      \if english
        It should be inserted or removed at the beginning and at the end.
      \endif
    
      \if german
        Es soll am Anfang und am Ende eingefügt bzw. entfernt werden.
      \endif
      */
      eBOTH
    };
    //-------------------------------------------------------------------------
    ///@{
    /**
    \if english
      Converts all uppercase letters of the passed string to lowercase.
      \param[in,out] rstr The reference to the string to be processed.
      \return The lowercase equivalent of rstr.
    \endif
  
    \if german
      Wandelt alle Großbuchstaben der übergebenen Zeichenkette in Kleinbuchstaben.
      \param[in,out] rstr Die Referenz auf die zu verarbeitende Zeichenkette.
      \return Das Kleinbuchstaben-Äquivalent von rstr.
    \endif
    */
    __tool_export_dll std::string toLower( const std::string & rstr );
    __tool_export_dll std::wstring toLower( const std::wstring & rstr );
    ///@}
    //-------------------------------------------------------------------------
    ///@{
    /**
    \if english
      Converts all lowercase letters of the passed string to uppercase.
      \param[in,out] rstr The reference to the string to be processed.
      \return The uppercase equivalent of rstr.
    \endif
  
    \if german
      Wandelt alle Kleinbuchstaben der übergebenen Zeichenkette in Großbuchstaben.
      \param[in,out] rstr Die Referenz auf die zu verarbeitende Zeichenkette.
      \return Das Großbuchstaben-Äquivalent von rstr.
    \endif
    */
    __tool_export_dll std::string toUpper( const std::string & rstr );
    __tool_export_dll std::wstring toUpper( const std::wstring & rstr );
    ///@}
    //-------------------------------------------------------------------------
    /*!
    \if english
      Convert the given integer number value to a hex string. <br>
      Found: https://newbedev.com/integer-to-hex-string-in-c
      \param[in] rValue The number value.
      \param[in] hex_len The size of the number value.
      \return The passed number value in the hexadecimal representation in a
        std::string.
    \endif
  
    \if german
      Liefert übergebenen Ganzzahl-Wert in eine Hex-String um. <br>
      Gefunden: https://newbedev.com/integer-to-hex-string-in-c
      \param[in] rValue Der Zahlwert.
      \param[in] hex_len Die Größe des Zahlwerts.
      \return Die übergebene Zahlwert in der hexadezimalen Darstellung in einem
        std::string.
    \endif
    */
   template <typename T> std::string toHexstr( const T & rValue, std::size_t hex_len = sizeof(T) << 1 )
   {
    static const char * t_chDigits = "0123456789ABCDEF";
    std::string         t_sRetVal( hex_len,'0' );

    for ( size_t i = 0, j = ( hex_len-1 )*4 ; i<hex_len; ++i,j-=4 )
    {
      t_sRetVal[i] = t_chDigits[(rValue>>j) & 0x0f];
    }
    return t_sRetVal;
} //-------------------------------------------------------------------------
    ///@{
    /**
    \if english
      Trim for std::string inspired by:

      Rodrigo C F Dias

      Software Engineering Degree

      Living in Natal, Brazil

      \sa http://www.codeproject.com/Articles/10880/A-trim-implementation-for-std-string

      \param[in,out] rstr Reference to the string to be freed from leading and 
        trailing characters.
      \param[in] c The character to be removed. If rstr == "XXXTestX" and c =='X'
        and ePos == eBOTH, rstr == becomes "Test".
      \param[in] ePos Specifies whether to remove at the beginning and/or the end.
    \endif
  
    \if german
      Trim für std::string inspiriert durch:

      Rodrigo C F Dias

      Software Engineering Degree

      Living in Natal, Brazil

      \sa http://www.codeproject.com/Articles/10880/A-trim-implementation-for-std-string

      \param[in,out] rstr Referenz auf den String, der von führenden und anhängenden
        Zeichen befreit werden soll.
      \param[in] c Das Zeichen welches entfernt werden soll. Wenn rstr == "XXXTestX"
        und c =='X' und ePos == eBOTH, wird aus rstr == "Test".
      \param[in] ePos Gibt an ob am Anfang und/oder am Ende entfernt werden soll.
    \endif
    */
    __tool_export_dll void trimString( std::string & rstr, char c = ' ', EnStrPos ePos = eBEFORE );
    __tool_export_dll void trimString( std::wstring & rstr, wchar_t c = ' ', EnStrPos ePos = eBEFORE );
    ///@}
    //-------------------------------------------------------------------------
    /*!
    \if english
      Checks the passed string whether it represents an integer.
      \param[in] rstr Reference to the string to be checked.
      \return If 'true', sthe passed string represents an integer. 
    \endif
  
    \if german
      Überprüft die übergebene Zeichenkette, ob sie eine ganze Zahl darstellt.
      \param[in] rstr Referenz auf die zu prüfende Zeichenkette.
      \return Wenn 'true', stellt die übergebene Zeichenkette eine ganze Zahl dar. 
    \endif
    */
    __tool_export_dll bool isAnInteger( const std::string & rstr );
    //-------------------------------------------------------------------------
    /*!
    \if english
      Fills the string passed with characters at the beginning or end.
      \param[in,out] rstr Reference to the string to be padded.
      \param[in] cFillItem The character to be padded.
      \param[in] nNumber The targeted string length.
      \param[in] ePos Specifies whether to fill at the beginning and/or at the end.
      \note StdStringFunctions::eBOTH is interpreted as StdStringFunctions::eBEHIND!
    \endif
  
    \if german
      Füllt übergebenen String am Anfang oder am Ende mit Zeichen auf.
      \param[in,out] rstr Referenz auf den String, der aufgefüllt werden soll.
      \param[in] cFillItem Das Zeichen mit dem aufgefüllt werden soll.
      \param[in] nNumber Die angestrebte Stringlänge.
      \param[in] ePos Gibt an ob am Anfang und/oder am Ende aufgefüllt werden soll.
      \note StdStringFunctions::eBOTH wird als StdStringFunctions::eBEHIND interpretiert!
    \endif
    */
    __tool_export_dll void fillString( std::string & rstr,
                                       char cFillItem,
                                       std::size_t nNumber,
                                       EnStrPos ePos );
    //-------------------------------------------------------------------------
    /*!
    \if english
      Fills a passed 0-terminated Char-Array with characters at the beginning or end.
      \param[in,out] pch Pointer to the 0-terminated char array to be added.
      \param[in] cFillItem The character to be filled with.
      \param[in] nNumber The targeted string length.
      \param[in] ePos Specifies whether to fill at the beginning or at the end.
      \return The resulting 0-terminated char array.
      \note StdStringFunctions::rBOTH is interpreted as StdStringFunctions::eBEHIND!
      \attention If a valid pointer is supplied, the calling instance must remove
        the char array from memory using delete[]!

    \endif
  
    \if german
      Füllt ein übergebenes 0-terminiertes Char-Array am Anfang oder am Ende mit Zeichen auf.
      \param[in,out] pch Zeiger auf das 0-terminierte Char-Array, welches aufgefüllt
        werden soll.
      \param[in] cFillItem Das Zeichen mit dem aufgefüllt werden soll.
      \param[in] nNumber Die angestrebte Stringlänge.
      \param[in] ePos Gibt an ob am Anfang oder am Ende aufgefüllt werden soll.
      \return Das rsultierende 0-terminierte Char-Array.
      \note StdStringFunctions::rBOTH wird als StdStringFunctions::eBEHIND interpretiert!.
      \attention Wenn ein gültiger Zeiger geliefert wird, muss die aufrufende Instanz
        das gelieferteChar-Array mittels delete[] aus dem Speicher entfernen!
    \endif
    */
    __tool_export_dll char * getFilledString( const char * pch,
                                              char cFillItem,
                                              std::size_t nNumber,
                                              EnStrPos ePos );
    //-------------------------------------------------------------------------
    /*!
    \if english
      Fills a passed std::string with characters at the beginning or end.
      \param[in,out] rstr String to be added.
      \param[in] cFillItem The character to be filled with.
      \param[in] nNumber The targeted string length.
      \param[in] ePos Specifies whether to fill at the beginning or at the end.
      \return The resulting 0-terminated char array.
      \note StdStringFunctions::rBOTH is interpreted as StdStringFunctions::eBEHIND!

    \endif
  
    \if german
      Füllt ein übergebenen std::string am Anfang oder am Ende mit Zeichen auf.
      \param[in,out] rstr Zeichenkett, welches aufgefüllt werden soll.
      \param[in] cFillItem Das Zeichen mit dem aufgefüllt werden soll.
      \param[in] nNumber Die angestrebte Stringlänge.
      \param[in] ePos Gibt an ob am Anfang oder am Ende aufgefüllt werden soll.
      \return Das rsultierende 0-terminierte Char-Array.
      \note StdStringFunctions::rBOTH wird als StdStringFunctions::eBEHIND interpretiert!.
    \endif
    */
    __tool_export_dll std::string getFilledString(  const std::string & rstr,
                                                    char cFillItem,
                                                    std::size_t nNumber,
                                                    EnStrPos ePos );
    //-------------------------------------------------------------------------
    /*!
    \if english
      Returns from a string all substrings, which are separated by a character, in
      one vector.
      \param[in] rstrSource The source string from which substrings are supplied.
      \param[in,out] rTokensVector The vector in which the substrings are delivered.
      \param[in] rstrDelimiters The character that divides the source string into substrings.
      \sa Taken from https://stackoverflow.com/questions/9823263/string-tokenization-in-c-including-delimiter-characters.
    \endif
  
    \if german
      Liefert aus einem String alle Teilstrings, welche durch ein Zeichen getrennt
      sind, in einem Vector.
      \param[in] rstrSource Der Quellstring aus dem Teistrings geliefert werden.
      \param[in,out] rTokensVector Der Vector, in dem die Teilstrings geliefert werden.
      \param[in] rstrDelimiters Das Zeichen, welche den Quellstring in Teistrings gliedert.
      \sa Aus https://stackoverflow.com/questions/9823263/string-tokenization-in-c-including-delimiter-characters übernommen.
    \endif
    */
    __tool_export_dll void tokenizeString( const std::string & rstrSource,
                                           std::vector<std::string> & rTokensVector,
                                           const std::string & rstrDelimiters = " " );
    //-------------------------------------------------------------------------
    /*!
    \if english
      Returns from a string all substrings, which are separated by a character, in
      one vector.
      \param[in] rstrSource The source string from which substrings are supplied.
      \param[in,out] rTokensVector The vector in which the substrings are delivered.
      \param[in] rcDelimiters The character that divides the source string into substrings.
      \sa Taken from https://stackoverflow.com/questions/9823263/string-tokenization-in-c-including-delimiter-characters.
    \endif
  
    \if german
      Liefert aus einem String alle Teilstrings, welche durch ein Zeichen getrennt
      sind, in einem Vector.
      \param[in] rstrSource Der Quellstring aus dem Teistrings geliefert werden.
      \param[in,out] rTokensVector Der Vector, in dem die Teistrings geliefert werden.
      \param[in] rcDelimiters Das Zeichen, welche den Quellstring in Teistrings gliedert.
      \sa Aus https://stackoverflow.com/questions/9823263/string-tokenization-in-c-including-delimiter-characters übernommen.
    \endif
    */
    __tool_export_dll void tokenizeString( const std::string & rstrSource,
                                           std::vector<std::string> & rTokensVector,
                                           const char & rcDelimiters = ' ' );
    //-------------------------------------------------------------------------
    /*!
    \if english
      Returns from a wstring all substrings, which are separated by a character,
      in one vector.
      \param[in] rwsSource The source string from which substrings are supplied.
      \param[in,out] rTokensVector The vector in which the substrings are delivered.
      \param[in] wcDelimiters The character that divides the source string into substrings.
    \endif
  
    \if german
      Liefert aus einem WString alle Teilstrings, welche durch ein Zeichen getrennt
      sind, in einem Vector.
      \param[in] rwsSource Der Quellstring aus dem Teistrings geliefert werden.
      \param[in,out] rTokensVector Der Vector, in dem die Teistrings geliefert werden.
      \param[in] wcDelimiters Das Zeichen, welche den Quellstring in Teistrings gliedert.
    \endif
    */
    __tool_export_dll void tokenizeString( const std::wstring & rwsSource,
                                           std::vector<std::wstring> & rTokensVector,
                                           wchar_t wcDelimiters = L' ' );
    //-------------------------------------------------------------------------
    ///@{
    /**
    \if english
      Inserts a string at the position passed, if not already present.
      \param[in,out] rsText In this string, the string rsIns is inserted at the
        position nPos passed, if not already present.
      \param[in] nPos The position in rsText where is checked and inserted if
        necessary.
      \param[in] rsIns The character string to be checked and inserted if necessary.
      \return If true, rsIns was inserted, otherwise nothing happened.
    \endif
  
    \if german
      Fügt eine Zeichenkette an übergebener Position ein, wenn noch nicht vorhanden.
      \param[in,out] rsText In diesem String wird an übergebener Position nPos
        die übergebene Zeichenkette rsIns eingefügt, wenn noch nicht vorhanden.
      \param[in] nPos Die Position in rsText wo geprüft und ggf. eingefügt wird.
      \param[in] rsIns Die zu prüfende und ggf. einzufügende Zeichenfolge.
      \return Wenn true, wurde rsIns eingefügt, ansonsten passierte nichts.
    \endif
    */
    __tool_export_dll bool insertIfNotAvailable(  std::string & rsText,
                                                  std::size_t nPos,
                                                  const std::string & rsIns );
    __tool_export_dll bool insertIfNotAvailable(  std::wstring & rsText,
                                                  std::size_t nPos,
                                                  const std::wstring & rsIns );
    ///@}
    //-------------------------------------------------------------------------
    ///@{
    /**
    \if english
      Exchanges all searched substrings in a string with another passed string.
      \param[in,out] rsText In this string, all strings from strFnd are removed
        and the string from strRep is inserted instead.
      \param[in] rsFnd The string to be exchanged.
      \param[in] rsRep The string to be inserted.
      \return A return value > 0 is the number of characters exchanged. A return
        value == 0 means that nothing was exchanged.
    \endif
  
    \if german
      Tauscht in einem String alle gesuchten Teilstrings mit einem anderen
      übergebenen String aus.
      \param[in,out] rsText In diesem String werden alle Zeichenfolgen aus strFnd
        entfernt und anstelle dieser die Zeichenfolge aus strRep eingefügt.
      \param[in] rsFnd Die auszutauschende Zeichenfolge.
      \param[in] rsRep Die einzufügende Zeichenfolge.
      \return Ein Rückgabwert > 0 ist die Anzahl der getauschten Zeichenfolgen.
        Ein Rückgabwert == 0 bedeutet, dass nichts getauscht wurde.
    \endif
    */
    __tool_export_dll std::size_t replaceAll( std::string & rsText,
                                              const std::string & rsFnd,
                                              const std::string & rsRep );
    __tool_export_dll std::size_t replaceAll( std::wstring & rsText,
                                              const std::wstring & rsFnd,
                                              const std::wstring & rsRep );
    ///@}
    //-------------------------------------------------------------------------
    /*!
    \if english
      Exchanges all searched characters in a string with another passed character.
      \param[in,out] rstrText In this string, all characters from pzOriginal are
        removed and the string from pzNew is inserted instead.
      \param[in] pzOriginal The character to be exchanged.
      \param[in] pzNew The character to be inserted.
      \return A return value > 0 is the number of characters exchanged. A return
        value == 0 means that nothing was exchanged.
    \endif

    \if german
      Tauscht in einem String alle gesuchten Zeichen mit einem anderen
      übergebenen Zeichen aus.
      \param[in,out] rstrText In diesem String werden alle Zeichen aus pzOriginal
        entfernt und anstelle dieses die Zeichenfolge aus pzNew eingefügt.
      \param[in] pzOriginal Das auszutauschende Zeichen.
      \param[in] pzNew Das einzufügende Zeichen.
      \return Ein Rückgabwert > 0 ist die Anzahl der getauschten Zeichen. Ein
        Rückgabwert == 0 bedeutet, dass nichts getauscht wurde.
    \endif
    */
    __tool_export_dll std::size_t replaceAll( std::string & rstrText,
                                              const char pzOriginal,
                                              const char pzNew );
    //-------------------------------------------------------------------------
    /*!
    \if english
      Exchanges all searched characters in a string with another passed character
      and enters the respective positions in passed vector.
      \param[in,out] rstrText In this string, all characters from pzOriginal are
        removed and the string from pzNew is inserted instead.
      \param[in,out] rPosVector The positions with the exchanged characters.
      \param[in] pzOriginal The character to be exchanged.
      \param[in] pzNew The character to be inserted.
      \return A return value > 0 is the number of characters exchanged. A return
        value == 0 means that nothing was exchanged.
    \endif

    \if german
      Tauscht in einem String alle gesuchten Zeichen mit einem anderen übergebenen
      Zeichen aus und trägt die jeweiligen Positionen in übergebenen Vector ein.
      \param[in,out] rstrText In diesem String werden alle Zeichen aus pzOriginal
        entfernt und anstelle dieses die Zeichenfolge aus pzNew eingefügt.
      \param[in,out] rPosVector Die Positionen mit den ausgetauschten Zeichen.
      \param[in] pzOriginal Das auszutauschende Zeichen.
      \param[in] pzNew Das einzufügende Zeichen.
      \return Ein Rückgabwert > 0 ist die Anzahl der getauschten Zeichen. Ein
        Rückgabwert == 0 bedeutet, dass nichts getauscht wurde.
    \endif
    */
    __tool_export_dll std::size_t replaceAll( std::string & rstrText,
                                              std::vector<std::size_t> & rPosVector,
                                              const char pzOriginal,
                                              const char pzNew );
    //-------------------------------------------------------------------------
    /*!
    \if english
      Converts a std::string into a std::wstring.
      \param[in] rsSourceString The std::string to be converted.
      \return The resulting std::wstring.
      \exception A "range_error" can be thrown when UTF-16 characters ate passed.
    \endif

    \if german
      Konvertiert einen std::string in einen std::wstring.
      \param[in] rsSourceString Der zu konvertierende std::string.
      \return Der resulrierende std::wstring.
      \exception Es kann bei Übergabe von UTF-16 Zeichen ein "range_error" geworfen
        werden
    \endif
    */
    __tool_export_dll std::wstring string2wstring( const std::string & rsSourceString );
    //-------------------------------------------------------------------------
    /*!
    \if english
      Converts a std::wstring into a std::string.
      \param[in] rwsSourceString The std::wstring to be converted.
      \return The resulting std::string.
      \note It is not convertetd to UTF-8 or any other code page, only the bytes
        are transferred.
    \endif

    \if german
      Konvertiert einen std::wstring in einen std::string.
      \param[in] rwsSourceString Der zu konvertierende std::wstring.
      \return Der resulrierende std::string.
      \note Es wird n icht in UTF-8 oder sonstige Code-Page gewandelt, sondern
        nur die Bytes übertragen.
    \endif
    */
    __tool_export_dll std::string wstring2string( const std::wstring & rwsSourceString );
    //-------------------------------------------------------------------------
    ///@{
    /**
    \if english
      Compares two std::string's. The search string can contain wildcards ('*' and '?').
    
      Written by Jack Handy - <A href="mailto:jakkhandy@hotmail.com">jakkhandy@hotmail.com</A>

      \sa https://www.codeproject.com/Articles/1088/Wildcard-string-compare-globbing
   
      Example:
      - StdStringFunctions::compareWildcardString( "g*k", "gee" ); -> NO equivalent
      - StdStringFunctions::compareWildcardString( "bl?h.*", "blah.jpg" ); -> Compliance
      - StdStringFunctions::compareWildcardString( "ge?ks*", "geeksforgeeks" ); -> Compliance
      - StdStringFunctions::compareWildcardString( "*pqrs", "pqrst" ); -> NO equivalent
      - StdStringFunctions::compareWildcardString( "abc*bcd", "abcdhghgbcd" ); -> Compliance
      - StdStringFunctions::compareWildcardString( "abc*c?", "abcd" ); -> NO equivalent
      - StdStringFunctions::compareWildcardString( "*c*d", "abcd" ); -> Compliance
      - StdStringFunctions::compareWildcardString( "*?c*d", "abcd" ); -> Compliance
      - StdStringFunctions::compareWildcardString( "123*67", "1234567" ); -> Compliance
      - StdStringFunctions::compareWildcardString( "12?*67", "1234567" ); -> Compliance
      - StdStringFunctions::compareWildcardString( "12??*67", "1234567" ); -> Compliance
      - StdStringFunctions::compareWildcardString( "12???*67", "1234567" ); -> Compliance
      - StdStringFunctions::compareWildcardString( "12????*67", "1234567" ); -> NO equivalent
      - StdStringFunctions::compareWildcardString( "1111", "1111" ); -> Compliance
      - StdStringFunctions::compareWildcardString( "*", "1111" ); -> Compliance
      - StdStringFunctions::compareWildcardString( "**", "1111" ); -> Compliance
      - StdStringFunctions::compareWildcardString( "σ*αία", "σημαία" ); -> Compliance
      - StdStringFunctions::compareWildcardString( "σ1*αία", "σημαία" ); -> NO equivalent

      \param[in] rsSearchString The search string with wildcards if necessary.
      \param[in] rsString The comparison string with which the search string is
        compared.
      \return If true, the search string corresponds to the comparison string
        passed.
    \endif

    \if german
      Vergleicht zwei std::string's. Der Suchstring kann Platzhaltern (wildcards
      '*' und '?') enthalten.
    
      Geschrieben von Jack Handy - <A href="mailto:jakkhandy@hotmail.com">jakkhandy@hotmail.com</A>

      \sa https://www.codeproject.com/Articles/1088/Wildcard-string-compare-globbing
   
      Beispiele:
      - StdStringFunctions::compareWildcardString( "g*k", "gee" ); -> KEINE Entsprechung
      - StdStringFunctions::compareWildcardString( "bl?h.*", "blah.jpg" ); -> Übereinstimmung
      - StdStringFunctions::compareWildcardString( "ge?ks*", "geeksforgeeks" ); -> Übereinstimmung
      - StdStringFunctions::compareWildcardString( "*pqrs", "pqrst" ); -> KEINE Entsprechung
      - StdStringFunctions::compareWildcardString( "abc*bcd", "abcdhghgbcd" ); -> Übereinstimmung
      - StdStringFunctions::compareWildcardString( "abc*c?", "abcd" ); -> KEINE Entsprechung
      - StdStringFunctions::compareWildcardString( "*c*d", "abcd" ); -> Übereinstimmung
      - StdStringFunctions::compareWildcardString( "*?c*d", "abcd" ); -> Übereinstimmung
      - StdStringFunctions::compareWildcardString( "123*67", "1234567" ); -> Übereinstimmung
      - StdStringFunctions::compareWildcardString( "12?*67", "1234567" ); -> Übereinstimmung
      - StdStringFunctions::compareWildcardString( "12??*67", "1234567" ); -> Übereinstimmung
      - StdStringFunctions::compareWildcardString( "12???*67", "1234567" ); -> Übereinstimmung
      - StdStringFunctions::compareWildcardString( "12????*67", "1234567" ); -> KEINE Entsprechung
      - StdStringFunctions::compareWildcardString( "1111", "1111" ); -> Übereinstimmung
      - StdStringFunctions::compareWildcardString( "*", "1111" ); -> Übereinstimmung
      - StdStringFunctions::compareWildcardString( "**", "1111" ); -> Übereinstimmung
      - StdStringFunctions::compareWildcardString( "σ*αία", "σημαία" ); -> Übereinstimmung
      - StdStringFunctions::compareWildcardString( "σ1*αία", "σημαία" ); -> KEINE Entsprechung

      \param[in] rsSearchString Der Suchstring ggf. mit Platzhaltern.
      \param[in] rsString Der Vergleichsstring, mit dem der Suchstring verglichen wird.
      \return Wenn true, entspricht der Suchestring dem übergebenen Vergleichsstring.
    \endif
    */
    __tool_export_dll bool compareWildcardString( const std::string & rsSearchString,
                                                  const std::string & rsString );
    __tool_export_dll bool compareWildcardString( const std::wstring & rsSearchString,
                                                  const std::wstring & rsString );
    ///@}
    //-------------------------------------------------------------------------
    ///@{
    /**
    \if english
      Outputs a string to the console in color.
      \param[in] rsCoutText The text to be output.
      \param[in] eForeGroundColor The foreground, i.e. text color.
      \param[in] eBackGroundColor The background color.
    \endif

    \if german
      Gibt auf der Konsole einen String farbig aus.
      \param[in] rsCoutText Der auszugebende Text.
      \param[in] eForeGroundColor Die Vordergrund- sprich Textfarbe.
      \param[in] eBackGroundColor Die Hinterdergrundfarbe.
    \endif
    */
    __tool_export_dll void colorConsoleCout(  const std::string & rsCoutText,
                                              EnCoutColor eForeGroundColor,
                                              EnCoutColor eBackGroundColor );
    __tool_export_dll void colorConsoleCout(  const std::wstring & rsCoutText,
                                              EnCoutColor eForeGroundColor,
                                              EnCoutColor eBackGroundColor );
    ///@}
    //-------------------------------------------------------------------------
    #if defined(_MSC_VER) || defined(__MINGW32__)
    /*!
    \if english
      Sets font to a color of the console to a color for the next output.
      \param[in] eForeGroundColor The foreground, i.e. text color.
      \param[in] eBackGroundColor The background color.
      \return The original CONSOLE_SCREEN_BUFFER_INFO attribute is needed to restore
        the previous state.
    \endif

    \if german
      Setzt die Schriftfarbe der Konsole auf eine Farbe für die nächste Ausgabe.
      \param[in] eForeGroundColor Die Vordergrund- sprich Textfarbe.
      \param[in] eBackGroundColor Die Hinterdergrundfarbe.
      \return Das ursprüngliche CONSOLE_SCREEN_BUFFER_INFO Attribute, wird benötigt
        um den ursprünglichen Zustand wieder herzustellen.
    \endif
    */
    __tool_export_dll WORD setColorConsoleCout( EnCoutColor eForeGroundColor,
                                                EnCoutColor eBackGroundColor );
    //^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
    #elif defined(__GNUC__)
    //vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv
    /*!
    \if english
      Sets font to a color of the console to a color for the next output.
      \param[in] eForeGroundColor The foreground, i.e. text color.
      \param[in] eBackGroundColor The background color.
    \endif

    \if german
      Setzt die Schriftfarbe der Konsole auf eine Farbe für die nächste Ausgabe.
      \param[in] eForeGroundColor Die Vordergrund- sprich Textfarbe.
      \param[in] eBackGroundColor Die Hinterdergrundfarbe.
    \endif
    */
    __tool_export_dll void setColorConsoleCout( EnCoutColor eForeGroundColor,
                                                EnCoutColor eBackGroundColor );
    #endif // defined(_MSC_VER) || defined(__MINGW32__) <> defined(__GNUC__)
    //-------------------------------------------------------------------------
    #if defined(_MSC_VER) || defined(__MINGW32__)
    /*!
    \if english
      Sets font to a color of the console to the default color for the next output.
      \param[in] wScreenBufferInfo The CONSOLE_SCREEN_BUFFER_INFO Attribute.
    \endif

    \if german
      Setzt die Schriftfarbe der Konsole auf Standardfarbe für die nächste Ausgabe.
      \param[in] wScreenBufferInfo Das CONSOLE_SCREEN_BUFFER_INFO Attribut.
    \endif
    */
    __tool_export_dll void resetColorConsoleCout( WORD wScreenBufferInfo );
    #endif // defined(_MSC_VER) || defined(__MINGW32__)
    //-------------------------------------------------------------------------
    #if defined(__GNUC__)
    /*!
    \if english
      Sets font to a color of the console to the default color for the next output.
    \endif

    \if german
      Setzt die Schriftfarbe der Konsole auf Standardfarbe für die nächste Ausgabe.
    \endif
    */
    __tool_export_dll void resetColorConsoleCout();
    #endif // defined(__GNUC__)
    //-------------------------------------------------------------------------
  }
  //---------------------------------------------------------------------------
  /*!
  \brief
  \if english
    Help functions for the processing of time.
  \endif

  \if german
    Hilfsfunktionen für die Verarbeitung von Zeit.
  \endif
  \copyright © 2010 Helmut Jakoby
  \author Helmut Jakoby
  */
  namespace TimeFunctions
  {
    //-------------------------------------------------------------------------
    /*!
    \if english
      Returns the 3 letter code of the passed month ordinal number.
      \param[in] uiMonth The month ordinal number is between 1 and 12.
      \return Der 3 letter code of the month..
      \exception eut::ErrorNException An exception will be thrown if the passed
        month ordinal number is outside the range of 1 to 12.
    \endif
  
    \if german
      Liefert den 3-Lettercode (in englisch) der Ordnungszahl des übergebenen Monats.
      \param[in] uiMonth Die Ordnungszahl des Monats liegt zwischen 1 und 12.
      \return Der 3-Lettercode des Monats.
      \exception eut::ErrorNException Es wird eine Exception geworfen, wenn die
        übergebene Monatsordinalzahl ausserhalb des Bereichs von 1-12 ist.
    \endif
    */
    __tool_export_dll std::string getMonth( unsigned int uiMonth );
    //-------------------------------------------------------------------------
    /*!
    \if english
      Returns passed date with time in military notation.
      day  time zone year
       |       |      |
      |03|1732|Z|jul|21|
           |      |
          time   month
      \param[in] rTime Date/time.
      \note The time zone is always 'Z'.
      \return See above.
    \endif
  
    \if german
      Liefert übergebenes Datum mit Zeit in der militärischen Notation. <br>
      Tag  Zeitzohne Jahr
       |       |      |
      |03|1732|Z|jul|21|
           |      |
        Uhrzeit Monat
      \param[in] rTime Datum/Zeit.
      \note Die Zeitzone ist immer 'Z'.
      \return s.o.
    \endif
    */
    __tool_export_dll std::string getMilitaryTime( const std::time_t & rTime );
    //-------------------------------------------------------------------------
    /*!
    \if english
      Returns the systemtime in military notation.
      day  time zone year
       |       |      |
      |03|1732|Z|jul|21|
           |      |
          time   month
      \note The time zone is always 'Z'.
      \return See above.
    \endif
  
    \if german
      Liefert die Systemzeit in der militärischen Notation. <br>
      Tag  Zeitzohne Jahr
       |       |      |
      |03|1732|Z|jul|21|
           |      |
        Uhrzeit Monat
      \note Die Zeitzone ist immer 'Z'.
      \return s.o.
    \endif
    */
    __tool_export_dll std::string getMilitarySystemTime();
    //-------------------------------------------------------------------------
    /*!
    \if english
      Returns the passed date with time formatted as defined in rsFormat (see
      description of std::strftime(...) ).
      \param[in] rTime Date/time.
      \param[in] rsFormat The date-time format (default is "%Y.%m.%d - %X".
      \return See above.
    \endif
  
    \if german
      Liefert die übergebene Datum mit Zeit formatiert wie in rsFormat definiert
      (siehe Beschreibung von std::strftime(...) ).
      \param[in] rTime Datum/Zeit.
      \param[in] rsFormat Das Datum- Zeit-Format (Voreinstellung ist "%Y.%m.%d - %X".
      \return s.o.
    \endif
    */
    __tool_export_dll std::string getTimeString( const std::time_t & rTime,
                                                 const std::string & rsFormat = "%Y.%m.%d - %X" );
    //-------------------------------------------------------------------------
    /*!
    \if english
      Returns the systemtime formatted as defined in rsFormat (see description
      of std::strftime(...) ).
      \param[in] rsFormat The date-time format (default is "%Y.%m.%d - %X".
      \return See above.
    \endif
  
    \if german
      Liefert die Systemzeit formatiert wie in rsFormat definiert (siehe
      Beschreibung von std::strftime(...) ).
      \param[in] rsFormat Das Datum- Zeit-Format (Voreinstellung ist "%Y.%m.%d - %X".
      \return s.o.
    \endif
    */
    __tool_export_dll std::string getSystemTimeString( const std::string & rsFormat = "%Y.%m.%d - %X" );
    //-------------------------------------------------------------------------
    /*!
    \if english
      Returns a time_t created from passed parameters.
      \param[in] uiYear The year.
      \param[in] uiMonth The month.
      \param[in] uiDay The day.
      \param[in] uiHour The hour.
      \param[in] uiMin The minute.
      \param[in] uiSec The second.
      \return See above.
    \endif
  
    \if german
      Liefert eine time_t erstellt aus übergebenen Parametern.
      \param[in] uiYear Das Jahr.
      \param[in] uiMonth Der Monat.
      \param[in] uiDay Der Tag.
      \param[in] uiHour Die Stunde.
      \param[in] uiMin Die Minute.
      \param[in] uiSec Die Sekunde.
      \return s.o.
    \endif
    */
    __tool_export_dll std::time_t getTime(  unsigned int uiYear,
                                            unsigned int uiMonth,
                                            unsigned int uiDay,
                                            unsigned int uiHour,
                                            unsigned int uiMin,
                                            unsigned int uiSec);
    //-------------------------------------------------------------------------
  }
  //---------------------------------------------------------------------------
} // namespace eut
#endif
