//-----------------------------------------------------------------------------
/*!
\file EuStdFunctions.cpp
\brief Source for StdStringFunctions
\author Helmut Jakoby
\copyright © 1995 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the EuTools module.

EuTools is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "EuStdFunctions.h"
#include <algorithm>
#include <locale>
#include <sstream>
#include <iostream>
#include <codecvt>
//-----------------------------------------------------------------------------
#ifdef _DEBUG_MEMORY_SPY
  #include <MilDebugNew.h>
#endif
//-----------------------------------------------------------------------------
namespace eut
{
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // Der Enumerationstyp "xxx" umfasst keine Bereichseinschränkung. Geben Sie "enum class" Vorrang vor "enum" (Enum.3)
    #pragma warning( disable : 26812 )
  #endif
  //---------------------------------------------------------------------------
  std::string StdStringFunctions::toLower( const std::string & rstr )
  {
    std::locale           t_Locale;
    std::ostringstream    t_osTemp;

    for ( std::string::size_type i = 0; i < rstr.length(); ++i )
    {
      t_osTemp << std::tolower( rstr[i], t_Locale );
    }
    return t_osTemp.str();
  }
  //---------------------------------------------------------------------------
  std::wstring StdStringFunctions::toLower( const std::wstring & rstr )
  {
    std::locale           t_Locale;
    std::wostringstream   t_osTemp;

    for ( std::string::size_type i = 0; i < rstr.length(); ++i )
    {
      t_osTemp << std::tolower( rstr[i], t_Locale );
    }
    return t_osTemp.str();
  }
  //---------------------------------------------------------------------------
  std::string StdStringFunctions::toUpper( const std::string & rstr )
  {
    std::locale           t_Locale;
    std::ostringstream    t_osTemp;

    for ( std::string::size_type i = 0; i < rstr.length(); ++i )
    {
      t_osTemp << std::toupper( rstr[i], t_Locale );
    }
    return t_osTemp.str();
  }
  //---------------------------------------------------------------------------
  std::wstring StdStringFunctions::toUpper( const std::wstring & rstr )
  {
    std::locale           t_Locale;
    std::wostringstream   t_osTemp;

    for ( std::string::size_type i = 0; i < rstr.length(); ++i )
    {
      t_osTemp << std::toupper( rstr[i], t_Locale );
    }
    return t_osTemp.str();
  }
  //---------------------------------------------------------------------------
  void StdStringFunctions::trimString( std::string & rstr, char c, EnStrPos ePos )
  {
    std::string::size_type pos = rstr.find_last_not_of( c );

    if ( pos != std::string::npos )
    {
    if ( ePos == eBEHIND || ePos == eBOTH )
    {
      rstr.erase( pos + 1 );
    }
    if ( ePos == eBEFORE || ePos == eBOTH )
    {
        pos = rstr.find_first_not_of( c );
        if ( pos != std::string::npos ) rstr.erase( 0, pos );
    }
    }
    else
    {
      rstr.erase( rstr.begin(), rstr.end() );
    }
  }
  //---------------------------------------------------------------------------
  void StdStringFunctions::trimString( std::wstring & rstr, wchar_t c, EnStrPos ePos )
  {
    std::wstring::size_type pos = rstr.find_last_not_of( c );

    if ( pos != std::string::npos )
    {
    if ( ePos == eBEHIND || ePos == eBOTH )
    {
      rstr.erase( pos + 1 );
    }
    if ( ePos == eBEFORE || ePos == eBOTH )
    {
        pos = rstr.find_first_not_of( c );
        if ( pos != std::string::npos ) rstr.erase( 0, pos );
    }
    }
    else
    {
      rstr.erase( rstr.begin(), rstr.end() );
    }
  }
  //---------------------------------------------------------------------------
  bool StdStringFunctions::isAnInteger( const std::string & rstr )
  {
    std::locale   t_Locale;

    for ( std::string::size_type i = 0; i < rstr.size(); ++i )
    {
      if ( std::isdigit( rstr[i], t_Locale ) == 0 )
      {
        return false;
      }
    }
    return true;
  }
  //---------------------------------------------------------------------------
  void StdStringFunctions::fillString(  std::string & rstr,
                                        char cFillItem,
                                        std::size_t nNumber,
                                        EnStrPos ePos)
  {
    std::string t_strRetVal;

    if ( rstr.length() < nNumber )
    {
      t_strRetVal.assign( nNumber - rstr.length(), cFillItem );
      if ( ePos == eBEFORE )
      {
        t_strRetVal += rstr;
        rstr = t_strRetVal;
      }
      else
      {
        rstr += t_strRetVal;
      }
    }
  }
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // This function or variable may be unsafe. Consider using strcpy_s instead.
    #pragma warning( disable : 4996 )
  #endif
  //---------------------------------------------------------------------------
  char * StdStringFunctions::getFilledString( const char * pch,
                                              char cFillItem,
                                              std::size_t nNumber,
                                              EnStrPos ePos )
  {
    if ( pch )
    {
      std::string   t_sPara1( pch ),
                    t_sRetVal = eut::StdStringFunctions::getFilledString( t_sPara1, cFillItem, nNumber, ePos );
      char  *       t_pchRetVal = new char[ t_sRetVal.size() + 1 ];

      strcpy( t_pchRetVal, t_sRetVal.c_str() );
      return t_pchRetVal;
    }
    return NULL_PTR;
  }
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // This function or variable may be unsafe. Consider using strcpy_s instead.
    #pragma warning( default : 4996 )
  #endif
  //---------------------------------------------------------------------------
  std::string StdStringFunctions::getFilledString(  const std::string & rstr,
                                                    char cFillItem,
                                                    std::size_t nNumber,
                                                    EnStrPos ePos )
  {
    std::string   t_sRetVal( rstr );
    eut::StdStringFunctions::fillString( t_sRetVal, cFillItem, nNumber, ePos );
    return t_sRetVal;
  }
  //---------------------------------------------------------------------------
  void StdStringFunctions::tokenizeString(  const std::string & rstrSource,
                                            std::vector<std::string> & rTokensVector,
                                            const std::string & rstrDelimiters )
  {
    // Skip rstrDelimiters at beginning.
    std::string::size_type lastPos = rstrSource.find_first_not_of( rstrDelimiters, 0 );
    // Find first "non-delimiter".
    std::string::size_type pos     = rstrSource.find_first_of( rstrDelimiters, lastPos );

    while ( std::string::npos != pos || std::string::npos != lastPos )
    {
      // Found a token, add it to the vector.
      rTokensVector.push_back( rstrSource.substr( lastPos, pos - lastPos ) );
      // Skip rstrDelimiters.  Note the "not_of"
      lastPos = rstrSource.find_first_not_of( rstrDelimiters, pos );
      // Find next "non-delimiter"
      pos = rstrSource.find_first_of( rstrDelimiters, lastPos );
    }
  }
  //---------------------------------------------------------------------------
  void StdStringFunctions::tokenizeString(  const std::string & rstrSource,
                                            std::vector<std::string> & rTokensVector,
                                            const char & rcDelimiters )
  {
    // Skip rcDelimiters at beginning.
    std::string::size_type lastPos = rstrSource.find_first_not_of(rcDelimiters, 0 );
    // Find first "non-delimiter".
    std::string::size_type pos     = rstrSource.find_first_of(rcDelimiters, lastPos );

    while ( std::string::npos != pos || std::string::npos != lastPos )
    {
      // Found a token, add it to the vector.
      rTokensVector.push_back( rstrSource.substr( lastPos, pos - lastPos ) );
      // Skip rcDelimiters.  Note the "not_of"
      lastPos = rstrSource.find_first_not_of(rcDelimiters, pos );
      // Find next "non-delimiter"
      pos = rstrSource.find_first_of(rcDelimiters, lastPos );
    }
  }
  //---------------------------------------------------------------------------
  void StdStringFunctions::tokenizeString(  const std::wstring & rwsSource, 
                                            std::vector<std::wstring> & rTokensVector, 
                                            wchar_t wcDelimiters )
  {
    std::wstring          t_wsTemp;
    std::wstringstream    t_wsStream( rwsSource );

    rTokensVector.clear();
    while ( std::getline( t_wsStream, t_wsTemp, wcDelimiters ) )
    {
      if ( ! t_wsTemp.empty() ) rTokensVector.push_back( t_wsTemp );
    }
  }
  //---------------------------------------------------------------------------
  bool StdStringFunctions::insertIfNotAvailable(  std::string & rsText,
                                                  std::size_t nPos,
                                                  const std::string & rsIns )
  {
    if ( rsText.substr( nPos, rsIns.size() ) != rsIns )
    {
      rsText.insert( nPos, rsIns );
      return true;
    }
    return false;
  }
  //---------------------------------------------------------------------------
  bool StdStringFunctions::insertIfNotAvailable(  std::wstring & rsText,
                                                  std::size_t nPos,
                                                  const std::wstring & rsIns )
  {
    if ( rsText.substr( nPos, rsIns.size() ) != rsIns )
    {
      rsText.insert( nPos, rsIns );
      return true;
    }
    return false;
  }
  //---------------------------------------------------------------------------
  size_t StdStringFunctions::replaceAll(  std::string & rsText,
                                          const std::string & rsFnd,
                                          const std::string & rsRep )
  {
    std::size_t   t_nCount( 0 );

    if ( ! rsText.empty() && ! rsFnd.empty() )
    {
      std::size_t  t_pos = rsText.find( rsFnd );

      while( t_pos != std::string::npos )
      {
        t_nCount++;
        rsText.replace( t_pos, rsFnd.length(), rsRep );
        t_pos = rsText.find( rsFnd, t_pos + rsRep.length() );
      }
    }
    return t_nCount;
  }
  //---------------------------------------------------------------------------
  size_t StdStringFunctions::replaceAll(  std::wstring & rsText,
                                          const std::wstring & rsFnd,
                                          const std::wstring & rsRep )
  {
    std::size_t   t_nCount( 0 );

    if ( ! rsText.empty() && ! rsFnd.empty() )
    {
      std::size_t  t_pos = rsText.find( rsFnd );

      while( t_pos != std::string::npos )
      {
        t_nCount++;
        rsText.replace( t_pos, rsFnd.length(), rsRep );
        t_pos = rsText.find( rsFnd, t_pos + rsRep.length() );
      }
    }
    return t_nCount;
  }
  //----------------------------------------------------------------------
  size_t StdStringFunctions::replaceAll(  std::string & rstrText,
                                          const char pzOriginal,
                                          const char pzNew )
  {
    std::size_t    t_iCount( 0 );

    if ( ! rstrText.empty() )
    {
      for ( std::size_t i = 0; i < rstrText.length(); ++i )
      {
        if ( rstrText[i] == pzOriginal )
        {
          t_iCount++;
          rstrText[i] = pzNew;
        }
      }
    }
    return t_iCount;
  }
  //----------------------------------------------------------------------
  size_t StdStringFunctions::replaceAll(  std::string & rstrText,
                                          std::vector<size_t> & rPosVector,
                                          const char pzOriginal,
                                          const char pzNew )
  {
    std::size_t    t_iCount( 0 );

    if ( ! rstrText.empty() )
    {
      for ( std::size_t i = 0; i < rstrText.length(); ++i )
      {
        if ( rstrText[i] == pzOriginal )
        {
          t_iCount++;
          rPosVector.push_back( i );
          rstrText[i] = pzNew;
        }
      }
    }
    return t_iCount;
  }
  //---------------------------------------------------------------------------
  #if defined _MSC_VER && __cplusplus >= 201703L
    // warning C4996: 'std::codecvt_utf8_utf16<wchar_t,1114111,(std::codecvt_mode)0>': warning STL4017: std::wbuffer_convert, std::wstring_convert, and the <codecvt> header (containing std::codecvt_mode, std::codecvt_utf8, std::codecvt_utf16, and std::codecvt_utf8_utf16) are deprecated in C++17.
    #pragma warning( disable : 4996 )
  #endif
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // Win64 -> Konvertierung von "size_t" nach "int", Datenverlust möglich
    #pragma warning( disable : 4267 )
  #endif
  //---------------------------------------------------------------------------
  std::wstring StdStringFunctions::string2wstring( const std::string & rsSourceString )
  {
    #if defined(__MINGW32__)  && __cplusplus >= 201103L
      int             t_iSizeNeeded = MultiByteToWideChar( CP_ACP, 0, & rsSourceString[0], (int) rsSourceString.size(), NULL_PTR, 0 );
      std::wstring    t_wsRetVal( t_iSizeNeeded, 0 );

      MultiByteToWideChar( CP_ACP, 0, & rsSourceString[0], (int) rsSourceString.size(), & t_wsRetVal[0], t_iSizeNeeded );
      return t_wsRetVal;
      //
      /*
      int       t_iSourceStringLength = rsSourceString.length() + 1;
      int       t_iWcBufferLen = MultiByteToWideChar( CP_ACP, 0, rsSourceString.c_str(), t_iSourceStringLength, 0, 0 );
      wchar_t * t_wcBuffer = new wchar_t[t_iWcBufferLen];

      MultiByteToWideChar( CP_ACP, 0, rsSourceString.c_str(), t_iSourceStringLength, t_wcBuffer, t_iSourceStringLength );
      std::wstring    t_wsRetVal( t_wcBuffer );
      delete[] t_wcBuffer;
      return t_wsRetVal;
      //*/
    #else // not defined(__MINGW32__) )  && __cplusplus >= 201103L
      std::wstring_convert<std::codecvt_utf8_utf16<wchar_t>> converter;

      std::wstring    t_wsRetVal = converter.from_bytes( rsSourceString );
      return t_wsRetVal;
    #endif // defined(__MINGW32__) && __cplusplus >= 201103L
  }
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // Win64 -> Konvertierung von "size_t" nach "int", Datenverlust möglich
    #pragma warning( default : 4267 )
  #endif
  //---------------------------------------------------------------------------
  std::string StdStringFunctions::wstring2string( const std::wstring & rwsSourceString )
  {
    ///*
    #if defined(__MINGW32__)  && __cplusplus >= 201103L
      int           t_iSizeNeeded2 = WideCharToMultiByte( CP_UTF8, 0, & rwsSourceString[0], (int)rwsSourceString.size(), NULL_PTR, 0, NULL_PTR, NULL_PTR );
      std::string   t_sRetVal( t_iSizeNeeded2, 0 );

      WideCharToMultiByte( CP_ACP, 0, & rwsSourceString[0], (int)rwsSourceString.size(), & t_sRetVal[0], t_iSizeNeeded2, NULL_PTR, NULL_PTR );
      return t_sRetVal;
    #else // not defined(__MINGW32__)  && __cplusplus >= 201103L
      std::wstring_convert<std::codecvt_utf8_utf16<wchar_t>> converter;

      std::string    t_sRetVal = converter.to_bytes( rwsSourceString );
      return t_sRetVal;
    #endif
    //*/
    //
    /*
      int           t_iSizeNeeded2 = WideCharToMultiByte(CP_UTF8, 0, &rwsSourceString[0], (int)rwsSourceString.size(), NULL_PTR, 0, NULL_PTR, NULL_PTR);
      std::string   t_sRetVal(t_iSizeNeeded2, 0);

      WideCharToMultiByte(CP_ACP, 0, &rwsSourceString[0], (int)rwsSourceString.size(), &t_sRetVal[0], t_iSizeNeeded2, NULL_PTR, NULL_PTR);
      return t_sRetVal;
    //*/
    //
    /*
      std::wstring_convert<std::codecvt_utf8_utf16<wchar_t>> converter;

      std::string    t_sRetVal = converter.to_bytes( rwsSourceString );
      return t_sRetVal;
    //*/
  }
  //---------------------------------------------------------------------------
  #if defined _MSC_VER && __cplusplus >= 201703L
    // warning C4996: 'std::codecvt_utf8_utf16<wchar_t,1114111,(std::codecvt_mode)0>': warning STL4017: std::wbuffer_convert, std::wstring_convert, and the <codecvt> header (containing std::codecvt_mode, std::codecvt_utf8, std::codecvt_utf16, and std::codecvt_utf8_utf16) are deprecated in C++17.
    #pragma warning( default : 4996 )
  #endif
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // "t_DEBUG_XXX": Lokale Variable ist initialisiert aber nicht referenziert
    #pragma warning( disable : 4189 )
  #endif
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: unused variable 't_DEBUG_XXX' [-Wunused-variable]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wunused-variable"
  #endif
  //---------------------------------------------------------------------------
  bool StdStringFunctions::compareWildcardString( const std::string & rsSearchString,
                                                  const std::string & rsString )
  {
    std::string   t_sSearchString( rsSearchString ),
                  t_sString( rsString );

    StdStringFunctions::trimString( t_sSearchString, ' ', eBOTH );
    StdStringFunctions::trimString( t_sString, ' ', eBOTH );
    // Written by Jack Handy - <A href="mailto:jakkhandy@hotmail.com">jakkhandy@hotmail.com</A>
    const char  * t_cp = NULL_PTR,
                * t_mp = NULL_PTR,
                * t_SearchString = t_sSearchString.c_str(),
                * t_String = t_sString.c_str();

    while ( ( * t_String ) && ( * t_SearchString != '*') )
    {
      if ( ( * t_SearchString != * t_String ) && ( * t_SearchString != '?' ) )
      {
        return false;
      }
      t_SearchString++;
      t_String++;
    }
    while ( * t_String )
    {
      if ( * t_SearchString == '*' )
      {
        if ( ! * ++t_SearchString )
        {
          return true;
        }
        t_mp = t_SearchString;
        t_cp = t_String + 1;
      }
      else if (( * t_SearchString == * t_String ) || ( * t_SearchString == '?' ) )
      {
      t_SearchString++;
      t_String++;
      }
      else
      {
        t_SearchString = t_mp;
        t_String = t_cp++;
      }
    }
    while ( * t_SearchString == '*' )
    {
      t_SearchString++;
    }
    #ifdef _DEBUG
      bool   t_DEBUG_RetVal = static_cast<bool>( ! * t_SearchString );
    #endif // _DEBUG
    return static_cast<bool>( ! * t_SearchString );
  }
  //---------------------------------------------------------------------------
  bool StdStringFunctions::compareWildcardString( const std::wstring & rsSearchString,
                                                  const std::wstring & rsString )
  {
    std::string   t_sSearchString( eut::StdStringFunctions::wstring2string( rsSearchString ) ),
                  t_sString( eut::StdStringFunctions::wstring2string( rsString ) );

    return StdStringFunctions::compareWildcardString( t_sSearchString, t_sString );
  }
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // "t_DEBUG_XXX": Lokale Variable ist initialisiert aber nicht referenziert
    #pragma warning( default : 4189 )
  #endif
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: unused variable 't_DEBUG_XXX' [-Wunused-variable]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // Konvertierung von "int" in "WORD", möglicher Datenverlust
    #pragma warning( disable : 4244 )
    // Die möglicherweise nicht initialisierte lokale Variable "t_CurrentConsoleAttribute" wurde verwendet.
    #pragma warning( disable : 4701 )
  #endif
  //---------------------------------------------------------------------------
  #if defined(__GNUC__) && ! defined (__clang__)
    // Warnung: 't_CurrentConsoleAttribute' may be used uninitialized in this function [-Wmaybe-uninitialized]
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wmaybe-uninitialized"
  #endif
  //---------------------------------------------------------------------------
  void StdStringFunctions::colorConsoleCout(  const std::string & rsCoutText,
                                              StdStringFunctions::EnCoutColor eForeGroundColor,
                                              StdStringFunctions::EnCoutColor eBackGroundColor )
  {
    #if defined(_MSC_VER) || defined(__MINGW32__)
      WORD  wDefaultSrceenBufferInfo = StdStringFunctions::setColorConsoleCout( eForeGroundColor, eBackGroundColor );

      std::cout << rsCoutText;
      StdStringFunctions::resetColorConsoleCout( wDefaultSrceenBufferInfo );
    #elif defined(__GNUC__)
      StdStringFunctions::setColorConsoleCout( eForeGroundColor, eBackGroundColor );
      std::cout << rsCoutText;
      StdStringFunctions::resetColorConsoleCout();
    #else // #if ! defined(_MSC_VER) && ! defined(__MINGW32__) && ! defined(__GNUC__)
      std::cout << rsCoutText;
    #endif // defined(_MSC_VER) || defined(__MINGW32__) <> defined(__GNUC__) <> _
  }
  //---------------------------------------------------------------------------
  void StdStringFunctions::colorConsoleCout(  const std::wstring & rsCoutText,
                                              StdStringFunctions::EnCoutColor eForeGroundColor,
                                              StdStringFunctions::EnCoutColor eBackGroundColor )
  {
    #if defined(_MSC_VER) || defined(__MINGW32__)
      WORD  wDefaultSrceenBufferInfo = StdStringFunctions::setColorConsoleCout( eForeGroundColor, eBackGroundColor );

      std::wcout << rsCoutText;
      StdStringFunctions::resetColorConsoleCout( wDefaultSrceenBufferInfo );
    #elif defined(__GNUC__)
      StdStringFunctions::setColorConsoleCout( eForeGroundColor, eBackGroundColor );
      std::wcout << rsCoutText;
      StdStringFunctions::resetColorConsoleCout();
    #else // #if ! defined(_MSC_VER) && ! defined(__MINGW32__) && ! defined(__GNUC__)
      std::cout << rsCoutText;
    #endif // defined(_MSC_VER) || defined(__MINGW32__) <> defined(__GNUC__) <> _
  }
  //---------------------------------------------------------------------------
  #if defined(_MSC_VER) || defined(__MINGW32__)
  WORD StdStringFunctions::setColorConsoleCout( EnCoutColor eForeGroundColor,
                                                EnCoutColor eBackGroundColor )
  {
    /*
    FOREGROUND_BLACK = 0;       //!< MS: xxxx 0000   Linux: \033[0;30;??m
    FOREGROUND_BLUE = 1;        //!< MS: xxxx 0001   Linux: \033[0;34;??m
    FOREGROUND_GREEN = 2;       //!< MS: xxxx 0010   Linux: \033[0;32;??m
    FOREGROUND_LIGHTBLUE = 3;   //!< MS: xxxx 0011   Linux: \033[0;36;??m
    FOREGROUND_RED = 4;         //!< MS: xxxx 0100   Linux: \033[0;31;??m
    FOREGROUND_MAGENTA = 5;     //!< MS: xxxx 0101   Linux: \033[0;35;??m
    FOREGROUND_YELLOW = 6;      //!< MS: xxxx 0110   Linux: \033[0;33;??m
    FOREGROUND_LIGHTGRAY = 7;   //!< MS: xxxx 0111   Linux: \033[0;39;??m
    FOREGROUND_GRAY = 8;        //!< MS: xxxx 1000   Linux: \033[0;37;??m
    FOREGROUND_WHITE = 15;      //!< MS: xxxx 1111   Linux: \033[0;39;??m

    BACKGROUND_BLACK = 0;       //!< MS: 0000 xxxx   Linux: \033[0;??;40m
    BACKGROUND_BLUE = 16;       //!< MS: 0001 xxxx   Linux: \033[0;??;44m
    BACKGROUND_GREEN = 32;      //!< MS: 0010 xxxx   Linux: \033[0;??;42m
    BACKGROUND_LIGHTBLUE = 48;  //!< MS: 0011 xxxx   Linux: \033[0;??;46m
    BACKGROUND_RED = 64;        //!< MS: 0100 xxxx   Linux: \033[0;??;41m
    BACKGROUND_MAGENTA = 80;    //!< MS: 0101 xxxx   Linux: \033[0;??;45m
    BACKGROUND_YELLOW = 96;     //!< MS: 0110 xxxx   Linux: \033[0;??;43m
    BACKGROUND_LIGHTGRAY = 112; //!< MS: 0111 xxxx   Linux: \033[0;??;47m
    BACKGROUND_GRAY = 128;      //!< MS: 1000 xxxx   Linux: \033[0;??;47m
    BACKGROUND_WHITE = 240;     //!< MS: 1111 xxxx   Linux: \033[0;??;47m
    */
    unsigned int    t_uiForgroundColor = 0,
                    t_uiBackgroundColor = 0;

    // Farben zurechtschieben
    if ( ! eForeGroundColor || eForeGroundColor == eCOLOR_WHITE )
    {
      t_uiForgroundColor = 15;
    }
    else
    {
      t_uiForgroundColor = eForeGroundColor - 1;
    }
    t_uiBackgroundColor = eBackGroundColor;
    if ( ! t_uiBackgroundColor )
    {
      ++t_uiBackgroundColor;
    }
    else if ( t_uiBackgroundColor == eCOLOR_WHITE )
    {
      t_uiBackgroundColor = 16;
    }
    --t_uiBackgroundColor;
    // shift nach links
    t_uiBackgroundColor = t_uiBackgroundColor << 4;

    HANDLE                        t_hConsole = GetStdHandle( STD_OUTPUT_HANDLE );
    WORD                          t_CurrentConsoleAttribute = 0;
    CONSOLE_SCREEN_BUFFER_INFO    t_ScrBufInf;

    if ( GetConsoleScreenBufferInfo( t_hConsole, & t_ScrBufInf ) )
    {
      t_CurrentConsoleAttribute = t_ScrBufInf.wAttributes;
    }
    SetConsoleTextAttribute( t_hConsole, t_uiForgroundColor | t_uiBackgroundColor );
    return t_CurrentConsoleAttribute;
  }
  //^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
  #elif defined(__GNUC__)
  //---------------------------------------------------------------------------
  void StdStringFunctions::setColorConsoleCout( EnCoutColor eForeGroundColor,
                                                EnCoutColor eBackGroundColor )
  {
    /*
    FOREGROUND_BLACK = 0;       //!< MS: xxxx 0000   Linux: \033[0;30;??m
    FOREGROUND_BLUE = 1;        //!< MS: xxxx 0001   Linux: \033[0;34;??m
    FOREGROUND_GREEN = 2;       //!< MS: xxxx 0010   Linux: \033[0;32;??m
    FOREGROUND_LIGHTBLUE = 3;   //!< MS: xxxx 0011   Linux: \033[0;36;??m
    FOREGROUND_RED = 4;         //!< MS: xxxx 0100   Linux: \033[0;31;??m
    FOREGROUND_MAGENTA = 5;     //!< MS: xxxx 0101   Linux: \033[0;35;??m
    FOREGROUND_YELLOW = 6;      //!< MS: xxxx 0110   Linux: \033[0;33;??m
    FOREGROUND_LIGHTGRAY = 7;   //!< MS: xxxx 0111   Linux: \033[0;39;??m
    FOREGROUND_GRAY = 8;        //!< MS: xxxx 1000   Linux: \033[0;37;??m
    FOREGROUND_WHITE = 15;      //!< MS: xxxx 1111   Linux: \033[0;39;??m

    BACKGROUND_BLACK = 0;       //!< MS: 0000 xxxx   Linux: \033[0;??;40m
    BACKGROUND_BLUE = 16;       //!< MS: 0001 xxxx   Linux: \033[0;??;44m
    BACKGROUND_GREEN = 32;      //!< MS: 0010 xxxx   Linux: \033[0;??;42m
    BACKGROUND_LIGHTBLUE = 48;  //!< MS: 0011 xxxx   Linux: \033[0;??;46m
    BACKGROUND_RED = 64;        //!< MS: 0100 xxxx   Linux: \033[0;??;41m
    BACKGROUND_MAGENTA = 80;    //!< MS: 0101 xxxx   Linux: \033[0;??;45m
    BACKGROUND_YELLOW = 96;     //!< MS: 0110 xxxx   Linux: \033[0;??;43m
    BACKGROUND_LIGHTGRAY = 112; //!< MS: 0111 xxxx   Linux: \033[0;??;47m
    BACKGROUND_GRAY = 128;      //!< MS: 1000 xxxx   Linux: \033[0;??;47m
    BACKGROUND_WHITE = 240;     //!< MS: 1111 xxxx   Linux: \033[0;??;47m
    */
    unsigned int    t_uiForgroundColor = 0,
                    t_uiBackgroundColor = 0;
    unsigned int    t_ForGrndColorArray[11] = { 9, 0, 4, 2, 6, 1, 5, 3, 9, 7, 9 },
                    t_BackGrndColorArray[11] = { 9, 0, 4, 2, 6, 1, 5, 3, 7, 7, 7 };

    t_uiForgroundColor = 30 + t_ForGrndColorArray[eForeGroundColor];
    t_uiBackgroundColor = 40 + t_BackGrndColorArray[eBackGroundColor];
    std::cout << "\033[0;" << t_uiForgroundColor << ";" << t_uiBackgroundColor << "m";
  }
  #endif // defined(_MSC_VER) || defined(__MINGW32__) <> defined(__GNUC__)
  //---------------------------------------------------------------------------
  #if defined(_MSC_VER) || defined(__MINGW32__)
  void StdStringFunctions::resetColorConsoleCout( WORD wScreenBufferInfo )
  {
    SetConsoleTextAttribute( GetStdHandle( STD_OUTPUT_HANDLE ), wScreenBufferInfo );
  }
  #endif // defined(_MSC_VER) || defined(__MINGW32__)
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
  void StdStringFunctions::resetColorConsoleCout()
  {
    std::cout << "\033[0m";
  }
  #endif // defined(__GNUC__)
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // Konvertierung von "int" in "WORD", möglicher Datenverlust
    #pragma warning( default : 4244 )
    // Die möglicherweise nicht initialisierte lokale Variable "t_CurrentConsoleAttribute" wurde verwendet.
    #pragma warning( default : 4701 )
  #endif
  //---------------------------------------------------------------------------
  #if defined(__GNUC__) && ! defined (__clang__)
    // Warnung: 't_CurrentConsoleAttribute' may be used uninitialized in this function [-Wmaybe-uninitialized]
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  std::string TimeFunctions::getMonth( unsigned int uiMonth )
  {
    #if __cplusplus >= 201103L
    std::vector<std::string>  t_Months{  "jan",
                                          "feb",
                                          "mar",
                                          "apr",
                                          "may",
                                          "jun",
                                          "jul",
                                          "aug",
                                          "sep",
                                          "oct",
                                          "nov",
                                          "dez" };
    #else // ifndef __cplusplus >= 201103L
    std::vector<std::string>  t_Months;

    t_Months.push_back( "jan" );
    t_Months.push_back( "feb" );
    t_Months.push_back( "mar" );
    t_Months.push_back( "apr" );
    t_Months.push_back( "may" );
    t_Months.push_back( "jun" );
    t_Months.push_back( "jul" );
    t_Months.push_back( "aug" );
    t_Months.push_back( "sep" );
    t_Months.push_back( "oct" );
    t_Months.push_back( "nov" );
    t_Months.push_back( "dez" );
    #endif // defined__cplusplus >= 201103L
    if ( uiMonth > 0 && uiMonth < 13 )
    {
      return t_Months[uiMonth];
    }
    throw eut::ErrorNException( "TimeFunctions::getMonth(unsigned int)", ERR_RANGE, __FILE__, __LINE__ ); 
  }
  //---------------------------------------------------------------------------
  std::string TimeFunctions::getMilitaryTime( const std::time_t & rTime )
  {
    struct std::tm      * t_pTmStruct;
    std::ostringstream    t_ossTemp;
    std::stringstream     t_strStreamYear,
                          t_strStreamMonth,
                          t_strStreamDay,
                          t_strStreamHour,
                          t_strStreamMinute;

    t_pTmStruct = std::localtime( & rTime );
    if ( rTime > -1 )
    {
      t_strStreamYear << t_pTmStruct->tm_year+1900;
      t_strStreamMonth << t_pTmStruct->tm_mon;
      t_strStreamDay << t_pTmStruct->tm_mday;
      t_strStreamHour << t_pTmStruct->tm_hour;
      t_strStreamMinute << t_pTmStruct->tm_min;

      t_ossTemp << eut::StdStringFunctions::getFilledString( t_strStreamDay.str(), '0', 2, eut::StdStringFunctions::eBEFORE )
                << eut::StdStringFunctions::getFilledString( t_strStreamHour.str(), '0', 2, eut::StdStringFunctions::eBEFORE )
                << eut::StdStringFunctions::getFilledString( t_strStreamMinute.str(), '0', 2, eut::StdStringFunctions::eBEFORE )
                << "Z"
                << eut::TimeFunctions::getMonth( t_pTmStruct->tm_mon )
                << t_strStreamYear.str().substr( t_strStreamYear.str().length()-2, 2 );
    }
    return t_ossTemp.str();
  }
  //---------------------------------------------------------------------------
  std::string TimeFunctions::getMilitarySystemTime()
  {
    std::time_t   t_now = std::time(0);

    return TimeFunctions::getMilitaryTime( t_now );
  }
  //---------------------------------------------------------------------------
  std::string TimeFunctions::getTimeString( const std::time_t & rTime,
                                            const std::string & rsFormat )
  {
    std::string       t_sRetval;
    struct std::tm    t_TmStruct = * std::localtime( & rTime );
    char              t_chBufer[100];

    if ( std::strftime( t_chBufer, sizeof( t_chBufer ), rsFormat.c_str(), &t_TmStruct) )
    {
      t_sRetval = t_chBufer;
    }
    return t_sRetval;
  }
  //---------------------------------------------------------------------------
  std::string TimeFunctions::getSystemTimeString( const std::string & rsFormat )
  {
    std::time_t   t_now = std::time(0);

    return TimeFunctions::getTimeString( t_now, rsFormat );
  }
  //---------------------------------------------------------------------------
  std::time_t TimeFunctions::getTime( unsigned int uiYear,
                                      unsigned int uiMonth,
                                      unsigned int uiDay,
                                      unsigned int uiHour,
                                      unsigned int uiMin,
                                      unsigned int uiSec)
  {
      std::time_t       t_TimeStamp;
      struct std::tm  * t_pTimeInfo;

      if ( uiYear < 1900 ){ uiYear = 1900; }
      else if ( uiYear > 2038 ){ uiYear = 2038; }
      if ( uiYear == 2038 )
      {
        if ( uiMonth < 1 || uiMonth > 1 ){ uiMonth = 1; }
        if ( uiMonth == 1 )
        {
          if ( uiDay > 18 ){ uiDay = 18; }
        }
      }
      if ( uiMonth < 1 || uiMonth > 11 ){ uiMonth = 1; }
      if ( uiDay < 1 || uiDay > 31 ){ uiDay = 1; }
      if ( uiHour > 23 ){ uiHour = 0; }
      if ( uiMin > 59 ){ uiMin = 0; }
      if ( uiSec > 59 ){ uiSec = 0; }
      std::time( & t_TimeStamp );
      t_pTimeInfo = std::localtime( & t_TimeStamp  );
      t_pTimeInfo->tm_year = uiYear - 1900;
      t_pTimeInfo->tm_mon = uiMonth - 1;
      t_pTimeInfo->tm_mday = uiDay;
      t_pTimeInfo->tm_hour = uiHour;
      t_pTimeInfo->tm_min = uiMin;
      t_pTimeInfo->tm_sec = uiSec;
      t_TimeStamp = mktime( t_pTimeInfo );
      return t_TimeStamp;
  }
  //---------------------------------------------------------------------------
  #ifdef _MSC_VER
    // Der Enumerationstyp "xxx" umfasst keine Bereichseinschränkung. Geben Sie "enum class" Vorrang vor "enum" (Enum.3)
    #pragma warning( default : 26812 )
  #endif
  //---------------------------------------------------------------------------
} // namespace eut
