#ifndef INC_PTLDVELEMENT_H
#define INC_PTLDVELEMENT_H

//-----------------------------------------------------------------------------
/*!
\file PtlDVElement.h
\brief
\if english
  Header for DVElement
\endif

\if german
  Header für DVElement
\endif
\author Helmut Jakoby
\copyright © 1995 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the PtlContainer module.

PtlContainer is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "PtlListTypes.h"
//-----------------------------------------------------------------------------
namespace ptl
{
  //---------------------------------------------------------------------------
  /*!
  \brief
  \if english
    Element of double linked lists.
    \details This class references an 'object' via a void-pointer. Additionally,
    a predecessor and a successor list element can be referenced.
  \endif

  \if german
    Element von doppelt verketteten Listen.
    \details Diese Klasse refenziert über einen void-pointer ein 'Objekt'.
      Zusätzlich können ein Vorläufer- und ein Nachläufer-Listenelement referenziert werden.
  \endif

  \copyright © 1995 Helmut Jakoby
  \author Helmut Jakoby
  */
  struct DVElement
  {
    public:
      //============== Attribute ============
      //-----------------------------------------------------------------------
      /*!
      \if english
        Pointer to the data object in the list.
      \endif
    
      \if german
        Zeiger auf das DatenObjekt in Liste.
      \endif
      */
      void * m_pData;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Pointer to next DVElement in list.
      \endif
    
      \if german
        Zeiger auf nächstes DVElement in Liste.
      \endif
      */
      DVElement * m_pNextElement;
      //-----------------------------------------------------------------------
      /*!
      \if english
        Pointer to previous DVElement in list.
      \endif
    
      \if german
        Zeiger auf vorherige DVElement in Liste.
      \endif
      */
      DVElement * m_pPreviousElement;
      //-----------------------------------------------------------------------
      //+ ============ Konstruktoren ============
      //-----------------------------------------------------------------------
      /*!
      \if english
        Constructor with parameter passing
        \param[in] pDataObject The object which is kept in a list by this DVElement; comes in #m_pData
        \param[in] pNext The following DVElement can be passed; comes in #m_pNextElement
        \param[in] pPrevious The preceding DVElement can be passed; comes in #m_pPreviousElement
      \endif
    
      \if german
        Konstruktor mit Parameterübergabe
        \param[in] pDataObject Das Objekt welches über dieses DVElement in einer
          Liste gehalten wird; kommt in #m_pData
        \param[in] pNext Das nachfolgende DVElement kann übergeben werden; kommt
          in #m_pNextElement
        \param[in] pPrevious Das davorliegende DVElement kann übergeben werden;
          kommt in #m_pPreviousElement
      \endif
      */
      DVElement (void* pDataObject, DVElement* pNext, DVElement* pPrevious);
  };
  //---------------------------------------------------------------------------
} // namespace ptl
#endif
