#ifndef INC_PtlLISTENFEHLER_H
#define INC_PtlLISTENFEHLER_H
/*!
\file PtlListErrors.h
\brief
\if english
  For each library, here 'PtlContainer' there is an error file.
\endif

\if german
  Für jede Bibliothek, hier 'PtlContainer' gibt es eine Fehler-Datei.
\endif
\author Helmut Jakoby
\copyright © 1995 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the PtlContainer module.

PtlContainer is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
namespace ptl
{
  //---------------------------------------------------------------------------
  /*!
  \enum ERRORS
  \brief
  \if english
    Error range = -15001 to -15100
  \endif

  \if german
    Fehlerbereich = -15001 bis -15100
  \endif
  */
  enum ERRORS
  {
    //-------------------------------------------------------------------------
    /*! -15001:
      \if english
        A  DVElement cannot be instantiated.
      \endif
    
      \if german
        Ein DVElement konte nicht instanziiert werden.
      \endif
      */
    INSTANTIATE_DVELEMENT = -15001,
    //-------------------------------------------------------------------------
    /*! -15002:
      \if english
        Error if an element to be inserted already exists in the container.
      \endif
    
      \if german
        Fehler wenn ein einzufügendes Element schon im Container vorhanden
        ist.
      \endif
      */
    OBJECT_ALREADY_AVAILABLE = -15002,
    //-------------------------------------------------------------------------
    /*! -15003:
      \if english
        An action was cancelled because there are no elements in the list.
      \endif
    
      \if german
        Eine Aktion wurde abgebrochen, weil es keine Elemente in der Liste gibt.
      \endif
      */
    ACTION_ON_EMPTY_LIST = -15003,
    //-------------------------------------------------------------------------
    /*! -15004:
      \if english
        There is no cached pointer to an element in the container.
      \endif
    
      \if german
        Es gibt keinen zwischengespeicherten Zeiger auf ein Element im Container.
      \endif
      */
    NO_RM_NULL = -15004,
    //-------------------------------------------------------------------------
    /*! -15005:
      \if english
        A buffered pointer is to become the current pointer, exists but not anymore.
      \endif
    
      \if german
        Ein zwischengespeicherten Zeiger soll zum aktuellen Zeiger werden, existiert
        aber nicht mehr.
      \endif
      */
    NO_RM_AWAY = -15005,
    //-------------------------------------------------------------------------
    /*! -15006:
      \if english
        An expected object is not in the container.
      \endif
    
      \if german
        Ein erwartetes Objekt ist nicht im Container.
      \endif
      */
    OBJEKT_UNAVAILABLE = -15006,
    //-------------------------------------------------------------------------
    /*! -15007:
      \if english
        No lower accounts could be instantiated.
      \endif
    
      \if german
        Es konnte kein unterer Konten instanziiert werden.
      \endif
      */
    INSTANTIATE_SUBNODE = -15007,
    //-------------------------------------------------------------------------
    /*! -15008:
      \if english
        No upper node present.
      \endif
    
      \if german
        Kein oberer Knoten vorhanden.
      \endif
      */
    NO_PARENT_NODE = -15008,
    //-------------------------------------------------------------------------
    /*! -15009:
      \if english
        The node ID is already assigned in the tree.
      \endif
    
      \if german
        Die Knoten-ID ist im Baum schon vergeben.
      \endif
      */
    DOUBLE_KNOTEN_ID = -15009,
    //-------------------------------------------------------------------------
    /*! -15010:
      \if english
         A string could not be instantiated.
      \endif
    
      \if german
        Ein string konnte nicht instatiiert werden.
      \endif
      */
    INSTANTIATE_STRING = -15010,
    //-------------------------------------------------------------------------
    /*! -15011:
      \if english
        An action could not be performed because the list is responsible for its
        objects.
      \endif
    
      \if german
        Eine Aktion konnte nicht durchgeführt werden, weil die Liste verantwortlich
        für ihre Objekte ist.
      \endif
      */
    IS_RESPONSIBLE = -15011,
    //-------------------------------------------------------------------------
    /*! -15012:
      \if english
        An attempt was made to copy a list to itself.
      \endif
    
      \if german
        Es wurde versucht eine Liste auf sich selbst zu kopieren.
      \endif
      */
    SAME_SOURCE_TARGET = -15012,
    //-------------------------------------------------------------------------
    /*! -15013:
      \if english
        An attempt was made to iterate beyond the end or beginning of the list.
      \endif
    
      \if german
        Es wurde versucht, über das Listen-Ende bzw. den Listen-Anfang hinaus zu
        iterieren.
      \endif
      */
    RANGE = -15013,
    //-------------------------------------------------------------------------
    /*! -15014:
      \if english
        No object was transferred.
      \endif
    
      \if german
        Es wurde kein Objekt übergeben.
      \endif
      */
    PASSED_NO_OBJECT = -15014,
    //-------------------------------------------------------------------------
    /*! -15015:
      \if english
        An object to be inserted already exists in the node tree.
      \endif
    
      \if german
        Ein einzufügendes Objekt ist schon im Knoten-Baum vorhanden.
      \endif
      */
    DOUBLE_NODE = -15015,
    //-------------------------------------------------------------------------
    /*! -15016:
      \if english
        An unrecoverable memory error has occurred.
      \endif
    
      \if german
        Ein nicht zu behebender Speicher-Fehler ist aufgetreten.
      \endif
      */
    FATAL_MEMORY_LEAK = -15016,
    //-------------------------------------------------------------------------
    /*! -15017:
      \if english
        A node to be inserted already exists in the tree above or below the node to be inserted.
      \endif
    
      \if german
        Ein einzufügender Knoten ist schon im Baum über oder unter dem einzufügenden
        Knoten vorhanden.
      \endif
      */
    CIRCULAR = -15017,
    //-------------------------------------------------------------------------
    /*! -15018:
      \if english
        The node to be inserted is not of the correct type.
      \endif
    
      \if german
        Der einzufügende Knoten ist nicht vom richtigen Typ.
      \endif
      */
    WRONG_NODETYPE = -15018
    //-------------------------------------------------------------------------
  };
  //---------------------------------------------------------------------------
} // namespace ptl
#endif
