//-----------------------------------------------------------------------------
/*!
\file PtlVDVList.cpp
\brief Source for VDVList
\author Helmut Jakoby
\copyright © 1995 Helmut Jakoby
<!-- @@TermsOfLicense BEGIN -->

<b>Terms of License</b>

This file is part of the PtlContainer module.

PtlContainer is free software:

This library is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
FITNESS FOR A PARTICULAR PURPOSE.

<b>GNU Affero General Public License Usage</b><br>
You can redistribute it and/or modify it under the terms of the GNU Affero General Public
License as published by the Free Software Foundation; either version 3 of the
License, or (at your option) any later version. Please review the following
information to ensure the GNU Affero General Public License requirements will be
met: https://www.gnu.org/licenses/agpl-3.0.en.html .
<!-- @@TermsOfLicense END -->
*/
//-----------------------------------------------------------------------------
#include "PtlVDVList.h"
#include "EuException.h"
//-----------------------------------------------------------------------------
namespace ptl
{
  //---------------------------------------------------------------------------
  VDVList::VDVList( ListMode eMode )
  : AVDVList(eMode),
    m_pCurrentDVElement(NULL_PTR),
    m_pTempCurrentDVElement(NULL_PTR)
  {

  }
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: base class 'class AVDVList' should be explicitly initialized in the copy constructor
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wextra"
  #endif
  //---------------------------------------------------------------------------
  VDVList::VDVList( const VDVList& rList )
  : m_pCurrentDVElement(NULL_PTR),
    m_pTempCurrentDVElement(NULL_PTR)
  {
      operator=( rList );
  }
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: unused parameter 'argc'
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  VDVList::~VDVList()
  {

  }
  //---------------------------------------------------------------------------
  int VDVList::addElementFrom( const VDVList & rList )
  {
    int        t_iRetVal = 0;
    VDVList * t_pUnConstParaListe = const_cast<VDVList*>(&rList);

    // Zeiger auf aktuelles Element zwischenspeichern hier und in übergebener Liste
    this->currentElementCache(M);
    t_pUnConstParaListe->currentElementCache(M);
    // selbige Methode in Oberklasse aufrufen
    t_iRetVal = AVDVList::_addElementFrom( rList );
    // wenn kein aktuelles Element vorhanden war...
    if (!m_pCurrentDVElement)
    {
      m_pCurrentDVElement = m_pFirstDVElement;
    }
    else
    {
      // wenn ein aktuelles Element da war; zurücksetzen.
      this->currentElementCache(RM);
    }
    // aktuelles Element in übergebener Liste zurücksetzen.
    t_pUnConstParaListe->currentElementCache(RM);
    return t_iRetVal;
  }
  //---------------------------------------------------------------------------
  void * VDVList::get( ElementPosition eElementPos )
  {
    bool    t_bNothingRetour = false;
    void *  t_pRetVal = NULL_PTR;

    switch (eElementPos)
    {
      case FIRST_POS:
      {
        // Aktuelles Element auf erstes Element setzen
        m_pCurrentDVElement = m_pFirstDVElement;
        break;
      }
      case LAST_POS:
      {
        // Aktuelles Element auf letztes Element setzen
        m_pCurrentDVElement = m_pLastDVElement;
        break;
      }
      case CURRENT_POS:
      {
        // hier machen wir nichts
        break;
      }
      case NEXT_POS:
      {
        // Wenn aktuelles Element und dessen nachfolgendes DVElement vorhanden,
        if ( m_pCurrentDVElement && ( ( m_pCurrentDVElement->m_pNextElement ) != NULL_PTR ) )
        {
          // mache dieses zum aktuellen DVElement
          m_pCurrentDVElement = m_pCurrentDVElement->m_pNextElement;
        }
        else
        {
          t_bNothingRetour = true;
        }
        break;
      }
      case PREVIOUS_POS:
      {
        // Wenn aktuelles Element und dessen vorgehendes DVElement vorhanden,
        if ( m_pCurrentDVElement && ( ( m_pCurrentDVElement->m_pPreviousElement ) != NULL_PTR ) )
        {
          // mache dieses zum aktuellen DVElement
          m_pCurrentDVElement = m_pCurrentDVElement->m_pPreviousElement;
        }
        else
        {
          t_bNothingRetour = true;
        }
        break;
      }
    } // ende switch
    // wenn aktuelles zugewiesen und s.o.
    if ( ( m_pCurrentDVElement ) && ( ! t_bNothingRetour ) )
    {
  // hole Datebobjekt vom aktuellen Element und gebe zurück
  t_pRetVal = m_pCurrentDVElement->m_pData;
    }
    // Ansonsten gebe kein pDataObject zurück
    return t_pRetVal;
  }

  //---------------------------------------------------------------------------
  int VDVList::get( void *& prRetVal, std::size_t nPosition, SeekMode eMode )
  {
    int  t_iErr = RANGE;

    prRetVal = NULL_PTR;
    switch (eMode)
    {
      case START:
      {
        t_iErr = get(prRetVal, nPosition, m_pFirstDVElement);
        break;
      }
      case END:
      {
        t_iErr = get(prRetVal, nPosition, m_pLastDVElement);
        break;
      }
      case CURRENT:
      {
        t_iErr = get(prRetVal, nPosition, m_pCurrentDVElement);
        break;
      }
    } // ende switch
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  int VDVList::get( void *& prRetVal, std::size_t nPosition, DVElement * pRefElement )
  {
    int           t_iErr = 0;
    std::size_t   t_nReferenceCounter = 0;
    DVElement   * t_pTempElement = NULL_PTR;

    t_pTempElement = pRefElement;
    // Wenn Richtung "nächstes Element"
    if ( nPosition > 0 )
    {
      // solange nächstes holen bis t_lReferenceCounter gleich nPosition oder Ende der Liste
      while ((t_nReferenceCounter != nPosition) && t_pTempElement)
      {
        t_pTempElement = t_pTempElement->m_pNextElement;
        t_nReferenceCounter++;
      }
    }
    /*
    // Wenn Richtung "vorheriges Element"
    else if ( nPosition < 0 )
    {
      // solange nächstes holen bis t_lReferenceCounter gleich nPosition oder Ende der Liste
      while ((t_nReferenceCounter != nPosition) && t_pTempElement)
      {
        t_pTempElement = t_pTempElement->m_pPreviousElement;
        t_nReferenceCounter--;
      }
    }
    */
    if (t_pTempElement)
    {
      m_pCurrentDVElement = t_pTempElement;
      prRetVal = t_pTempElement->m_pData;
    }
    else
    {
      prRetVal = NULL_PTR;
      t_iErr = RANGE;
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  int VDVList::insert( void* pDataObject, InsertPosition eWhere, VH eBeforeBehind )
  {
    DVElement * t_pRefElement = NULL_PTR;
    int          t_iErr = 0;

    switch (eWhere)
    {
      case InsertHEAD:
      {
        t_pRefElement = m_pFirstDVElement;
        break;
      }
      case InsertLAST:
      {
        t_pRefElement = m_pLastDVElement;
        break;
      }
      case InsertCURRENT:
      {
        t_pRefElement = m_pCurrentDVElement;
        break;
      }
    }
    if (pDataObject)    // weiter nur wenn gültiges pDataObject übergeben wurde
    {
      // einzufügendes Element wird zurückgeliefert
      m_pCurrentDVElement = insertSrc(  pDataObject,
                                        t_pRefElement,
                                        eBeforeBehind);
      // Wenn einfügen nicht klappte; kein neues DVElement geliefert wird...
      if (!m_pCurrentDVElement)
      {
        // Wenn Fehler, dann instanziierung nicht geklappt
        if (m_iErrorCode)
        {
          t_iErr = INSTANTIATE_DVELEMENT; // instanziieren von DVElement ging nicht
        }
        else
        {
          t_iErr = OBJECT_ALREADY_AVAILABLE;  // Kein Einfügen; Datenobjekt schon drinn (nur bei UNIQUE)
        }
      }
    }
    else
    {
      t_iErr = PASSED_NO_OBJECT;        // kein pDataObject übergeben
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: enumeration value 'NEXT_POS' not handled in switch
    // Warnung: enumeration value 'PREVIOUS_POS' not handled in switch
    #pragma GCC diagnostic push
    #pragma GCC diagnostic ignored "-Wswitch"
  #endif
  //---------------------------------------------------------------------------
  int VDVList::remove( ElementPosition eElementPos )
  {
    int  t_iErr = 0;

    switch (eElementPos)
    {
      case FIRST_POS:
      {
        // Aktuelles Element auf erstes Element setzen
        m_pCurrentDVElement = m_pFirstDVElement;
        break;
      }
      case LAST_POS:
      {
        // Aktuelles Element auf letztes Element setzen
        m_pCurrentDVElement = m_pLastDVElement;
        break;
      }
      case CURRENT_POS:
      {
        // hier machen wir nichts
        break;
      }
    } // ende switch
    // Wenn aktuelles Element vorhanden,
    if (m_pCurrentDVElement)
    {
      m_pCurrentDVElement = removeSrc(m_pCurrentDVElement);
    }
    else
    {
      // Liste schon leer
      t_iErr = ACTION_ON_EMPTY_LIST;
    }
    return t_iErr;
  }
  //---------------------------------------------------------------------------
  #if defined(__GNUC__)
    // Warnung: enumeration value 'NEXT_POS' not handled in switch
    // Warnung: enumeration value 'PREVIOUS_POS' not handled in switch
    #pragma GCC diagnostic pop
  #endif
  //---------------------------------------------------------------------------
  void VDVList::removeAll()
  {
      // Alle löschen
      AVDVList::removeAll();
      // und aktuelles auf 0
      m_pCurrentDVElement = NULL_PTR;
  }
  //---------------------------------------------------------------------------
  int VDVList::currentElementCache( ElementCache eWhich )
  {
    switch ( eWhich )
    {
      case M:  // wenn aktueller Zeiger gespeichert werden soll...
      {
        m_pTempCurrentDVElement = m_pCurrentDVElement;
        return 0;
      } // Ende case M:
      case RM:
      {
        // wenn alter Zeiger = 0 und neuer <> 0, dann darf kein RM durchgeführt werden
        if (!m_pTempCurrentDVElement && m_pCurrentDVElement)
        {
          return NO_RM_NULL;
        }
        // wenn alter Zeiger != 0 und nicht mehr in Liste, kein RM möglich
        if (m_pTempCurrentDVElement && (!getNumberDataObjects(m_pTempCurrentDVElement->m_pData)))
        {
          return NO_RM_AWAY;
        }
        // wenn bis jetzt nicht raus, dann alles 0
        m_pCurrentDVElement = m_pTempCurrentDVElement;
      } // Ende case RM:
    } // ENDE switch (eWhich)
    return 0;
  }
  //---------------------------------------------------------------------------
  int VDVList::setCurrentPosition( void* pDataObject )
  {
    // Wenn DatenObjekt übergeben
    if ( pDataObject )
    {
      void  *  t_pComparePointer = NULL_PTR;  // VoidPointer zum Vergleichen

      // Aktuelles Element aufbewahren, falls übergebenes DatenObjekt nicht gefunden
      this->currentElementCache(M);
      // Wenn erstes Element vorhanden
      if ( ( t_pComparePointer = get( FIRST_POS ) ) != NULL_PTR )
      {
        do  // solange bis gefunden
        {
          if (t_pComparePointer == pDataObject)
          {
            return 0;
          }
        } while ( ( t_pComparePointer = get( NEXT_POS ) ) != NULL_PTR );
        // wenn hier, dann ist pDataObject nicht in Liste
        // Aktuellen ElementZeiger zurücksetzen...
        this->currentElementCache(RM);
        // und mit Info zurück
        return OBJEKT_UNAVAILABLE;
      }
      else
      {
        return ACTION_ON_EMPTY_LIST;
      }
    }
    return PASSED_NO_OBJECT;  // kein pDataObject übergeben
  }
  //---------------------------------------------------------------------------
  VDVList & VDVList::operator= ( const VDVList& rList )
  {
    if (this != &rList)
    {
      // alle Element löschen um neue Elemente zu übertragen
      this->removeAll();
      int   t_iErr = this->addElementFrom(rList);
      if ( t_iErr )
      {
        throw eut::ErrorNException("Zufügen der Elemente gescheitert!", t_iErr, __FILE__, __LINE__);
      }
    }
    return *this;
  }
  //---------------------------------------------------------------------------
} // namespace ptl
